"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClaimService = void 0;
const common_1 = require("@nestjs/common");
const ExcessGeneration_repository_1 = require("./repositories/ExcessGeneration/ExcessGeneration.repository");
const LeftoverConsumption_repository_1 = require("./repositories/LeftoverConsumption/LeftoverConsumption.repository");
const MatchResult_repository_1 = require("./repositories/MatchResult/MatchResult.repository");
const origin_247_certificate_1 = require("@energyweb/origin-247-certificate");
const lodash_1 = require("lodash");
let ClaimService = class ClaimService {
    constructor(matchResultRepo, leftoverConsumptionRepo, excessGenerationRepo, certificateService) {
        this.matchResultRepo = matchResultRepo;
        this.leftoverConsumptionRepo = leftoverConsumptionRepo;
        this.excessGenerationRepo = excessGenerationRepo;
        this.certificateService = certificateService;
    }
    async claim(input) {
        const { algorithmFn, data, timeframe, claimCustomizationFn } = input;
        const matchingResult = algorithmFn(data);
        const filteredMatchingResult = this.filterZeroVolumeMatches(matchingResult);
        const resultsForClaim = claimCustomizationFn(filteredMatchingResult.matches);
        await this.certificateService.batchClaim(resultsForClaim);
        const raport = this.createRaport(filteredMatchingResult, timeframe);
        await this.createLogs(raport);
        return raport;
    }
    createRaport(matchingResult, timeframe) {
        const mappedMatches = matchingResult.matches.map((m) => ({
            volume: m.volume.toString(),
            consumerId: m.consumption.id,
            consumerMetadata: lodash_1.omit(m.consumption, 'volume', 'consumerId'),
            generatorId: m.generation.id,
            generatorMetadata: lodash_1.omit(m.generation, 'volume', 'generatorId'),
            timestamp: timeframe.to
        }));
        const mappedLeftoverConsumptions = matchingResult.leftoverConsumptions.map((lc) => ({
            consumerId: lc.id,
            consumerMetadata: lc,
            volume: lc.volume.toString(),
            timestamp: timeframe.to
        }));
        const mappedExcessGenerations = matchingResult.excessGenerations.map((eg) => ({
            generatorId: eg.id,
            generatorMetadata: eg,
            volume: eg.volume.toString(),
            timestamp: timeframe.to
        }));
        return {
            matches: mappedMatches,
            leftoverConsumptions: mappedLeftoverConsumptions,
            excessGenerations: mappedExcessGenerations
        };
    }
    async createLogs(raport) {
        await Promise.all([
            this.createMatchResultLogs(raport.matches),
            this.createLeftoverConsumptionLogs(raport.leftoverConsumptions),
            this.createExcessGenerationLogs(raport.excessGenerations)
        ]);
    }
    async createMatchResultLogs(matchResults) {
        await Promise.all(matchResults.map(async (mr) => {
            await this.matchResultRepo.create(mr);
        }));
    }
    async createLeftoverConsumptionLogs(leftoverConsumptions) {
        await Promise.all(leftoverConsumptions.map(async (lc) => {
            await this.leftoverConsumptionRepo.create(lc);
        }));
    }
    async createExcessGenerationLogs(excessGenerations) {
        await Promise.all(excessGenerations.map(async (eg) => {
            await this.excessGenerationRepo.create(eg);
        }));
    }
    filterZeroVolumeMatches(matchingResult) {
        return {
            matches: matchingResult.matches.filter((m) => m.volume.gt(0)),
            leftoverConsumptions: matchingResult.leftoverConsumptions.filter((lc) => lc.volume.gt(0)),
            excessGenerations: matchingResult.excessGenerations.filter((eg) => eg.volume.gt(0))
        };
    }
};
ClaimService = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject(MatchResult_repository_1.MATCH_RESULT_REPOSITORY)),
    __param(1, common_1.Inject(LeftoverConsumption_repository_1.LEFTOVER_CONSUMPTION_REPOSITORY)),
    __param(2, common_1.Inject(ExcessGeneration_repository_1.EXCESS_GENERATION_REPOSITORY)),
    __metadata("design:paramtypes", [Object, Object, Object, origin_247_certificate_1.OffChainCertificateService])
], ClaimService);
exports.ClaimService = ClaimService;
//# sourceMappingURL=claim.service.js.map